#include <actions.h>
#include <gears.h>
#include <debug.h>
#include <string.h>
#include <browser.h>
#include <packagekit/pk-enum.h>

extern Ecore_Evas * ee;
struct Installer_Progressbar *_AP_;

static char * progress_bar_type_text (enum ActionType type) {
	if (type==INSTALL ||
			type==UPDATE) {
		return "Downloading ";
	}else if (type==UNINSTALL) {
		return "Uninstalling ";

	}else {
		return "";
	}
}

static void pk_update_progress_cb (void *data, DBusMessage *message);
static void pk_change_status_cb (void *data, DBusMessage *message);
static void pk_error_cb (void *data, DBusMessage *message);
static void pk_message_cb (void *data, DBusMessage *message);
static void __delayed_fade_out(void *data, DBusMessage *message);

void action(struct Target *target) {
	Pk_Operation * pko;
	Evas *evas = ecore_evas_get(ee);

	if (!target) {
		error("What is your target!!\n");
		return;
	}
	pko=(Pk_Operation *)malloc(sizeof(Pk_Operation));
	memset(pko,0,sizeof(Pk_Operation));

	pko->payload = target;

	// Adding general call back here
	pko->Operations[ProgressChanged_sig] = &pk_update_progress_cb;
	pko->Operations[StatusChanged_sig] = &pk_change_status_cb;


	_AP_=new_Progressbar(evas,progress_bar_type_text(target->type));

	// Delay fade out call back
	e_dbus_proxy_connect_signal(pk_proxy(),"Finished", &__delayed_fade_out, pko);

	if (target->type==INSTALL) {
		debug("Do install!\n");
		pk_install_package(pko,target->package);
	}else if (target->type==UPDATE) {
		debug("Do update!\n");
		pk_update_package(pko,target->package);
	}else if (target->type==UNINSTALL) {
		debug("Do uninstall!\n");
		pk_uninstall_package(pko,target->package);
	}else {
		error("Hey!! Bad type %d!!\n",target->type);
		free_Progressbar(_AP_);
		free(pko);
		_AP_=NULL;
		return;
	}
}


static void pk_update_progress_cb (void *data, DBusMessage *message) {
	Pk_Operation * pko=(Pk_Operation *)data;
	struct Target * target = (struct Target *)pko->payload;
	DBusError _err;
	char *tid;
	unsigned int percentage, subpercentage, elapsed, remaining;
	debug("get progress cb!!\n");
	dbus_error_init(&_err);
	if (!dbus_message_get_args(message,&_err,
				DBUS_TYPE_STRING, &tid,
				DBUS_TYPE_UINT32, &percentage,
				DBUS_TYPE_UINT32, &subpercentage,
				DBUS_TYPE_UINT32, &elapsed,
				DBUS_TYPE_UINT32, &remaining,
				DBUS_TYPE_INVALID)) {
		error("Error Message: %s\n",_err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if(!pk_should_proxy(pko,tid)) {
		return;
	}
	target->progress= percentage;	
	debug("Progress: %d %d %d %d\n",percentage,subpercentage,elapsed,remaining);
	if (_AP_) {
		progressbar_update(_AP_,target->progress);
	}
}

static int progressbar_delay_die(void *data) {
	if (_AP_) {
		free_Progressbar(_AP_);
		_AP_=NULL;
	}
	show_categories_ilist();
	return 0;
}

//pk_marshal_VOID__STRING_STRING change_status
static void pk_change_status_cb (void *data, DBusMessage *message) {
	char *tid;
	char *status_text;
	Pk_Operation * pko=(Pk_Operation *)data;
	struct Target * target = (struct Target *)pko->payload;
	DBusError _err;
	dbus_error_init(&_err);
	if (!dbus_message_get_args(message,&_err,
				DBUS_TYPE_STRING, &tid,
				DBUS_TYPE_STRING, &status_text,
				DBUS_TYPE_INVALID )) {
		error("Error Message: '%s'\n",_err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if(!pk_should_proxy(pko,tid)) {
		return;
	}
	debug("Status Changed: %s\n",status_text);
	if (!_AP_) {
		error("_AP_ is NULL!!\n");
		return;
	}
	if (target->type==INSTALL) {
		if (pk_status_enum_from_text(status_text)==PK_STATUS_ENUM_DOWNLOAD) {		
			progressbar_text_set(_AP_,"Downloading ");
		} else if (pk_status_enum_from_text(status_text)==PK_STATUS_ENUM_INSTALL) {
			progressbar_update(_AP_,100);
			progressbar_text_set(_AP_,"Installing...");
		} else {
			error("Hun? %s!!\n",status_text);
		}
	}else if (target->type==UPDATE) {
		if (pk_status_enum_from_text(status_text)==PK_STATUS_ENUM_DOWNLOAD) {		
			progressbar_text_set(_AP_,"Downloading ");
		} else if (pk_status_enum_from_text(status_text)==PK_STATUS_ENUM_UPDATE) {
			progressbar_update(_AP_,100);
			progressbar_text_set(_AP_,"Updating...");
		} else {
			error("Hun? %s!!\n",status_text);
		}
	}else if (target->type==UNINSTALL) {
			progressbar_update(_AP_,100);
	}
}

static void __delayed_fade_out(void *data, DBusMessage *message) {
	char *tid;
	char *exit_text;
	unsigned int runtime;
	int i;
	Pk_Operation * pko=(Pk_Operation *)data;
	DBusError _err;
	dbus_error_init(&_err);
	if (!dbus_message_get_args(message,&_err,
				DBUS_TYPE_STRING, &tid,
				DBUS_TYPE_STRING, &exit_text,
				DBUS_TYPE_UINT32,   &runtime, 
				DBUS_TYPE_INVALID )) {
		error("Error Message: '%s'\n",_err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if(!pk_should_proxy(pko,tid)) {
		return;
	}
	debug("Transaction '%s' is finished!\n",pko->tid);
	debug("exit_text: %s\n",exit_text);
	debug("runtime %d\n",runtime);

	ecore_timer_add(3,&progressbar_delay_die,NULL);
	
	return;
}
